@@ -2,12 +2,14 @@
 //! Functions for 64bit floating point number based vectors. Please refer to the other chapters of the help for documentation of the functions.
 use super::*;
 use super::super::*;
+use super::super::combined_ops::*;
 use window_functions::*;
 use conv_types::*;
 use num::complex::Complex64;
 use std::slice;
 use std::os::raw::c_void;
 use std::mem;
+use std::sync::Arc;
 
 #[no_mangle]
 pub extern fn delete_vector64(vector: Box<DataVector64>) {
@@ -23,7 +25,7 @@ pub extern fn new64(is_complex: i32, domain: i32, init_value: f64, length: usize
             DataVectorDomain::Frequency
         };
         
-	let vector = Box::new(DataVector64::new(is_complex != 0, domain, init_value, length, delta));
+    let vector = Box::new(DataVector64::new(is_complex != 0, domain, init_value, length, delta));
     vector
 }
 
@@ -36,7 +38,7 @@ pub extern fn new_with_performance_options64(is_complex: i32, domain: i32, init_
             DataVectorDomain::Frequency
         };
         
-	let vector = Box::new(DataVector64::new_with_options(is_complex != 0, domain, init_value, length, delta, MultiCoreSettings::new(core_limit, early_temp_allocation)));
+    let vector = Box::new(DataVector64::new_with_options(is_complex != 0, domain, init_value, length, delta, MultiCoreSettings::new(core_limit, early_temp_allocation)));
     vector
 }
 
@@ -207,28 +209,28 @@ pub extern fn root64(vector: Box<DataVector64>, value: f64) -> VectorResult<Data
 }
 
 #[no_mangle]
-pub extern fn power64(vector: Box<DataVector64>, value: f64) -> VectorResult<DataVector64> {
-    convert_vec!(vector.power(value))
+pub extern fn powf64(vector: Box<DataVector64>, value: f64) -> VectorResult<DataVector64> {
+    convert_vec!(vector.powf(value))
 }
 
 #[no_mangle]
-pub extern fn logn64(vector: Box<DataVector64>) -> VectorResult<DataVector64> {
-    convert_vec!(vector.logn())
+pub extern fn ln64(vector: Box<DataVector64>) -> VectorResult<DataVector64> {
+    convert_vec!(vector.ln())
 }
 
 #[no_mangle]
-pub extern fn expn64(vector: Box<DataVector64>) -> VectorResult<DataVector64> {
-    convert_vec!(vector.expn())
+pub extern fn exp64(vector: Box<DataVector64>) -> VectorResult<DataVector64> {
+    convert_vec!(vector.exp())
 }
 
 #[no_mangle]
-pub extern fn log_base64(vector: Box<DataVector64>, value: f64) -> VectorResult<DataVector64> {
-    convert_vec!(vector.log_base(value))
+pub extern fn log64(vector: Box<DataVector64>, value: f64) -> VectorResult<DataVector64> {
+    convert_vec!(vector.log(value))
 }
 
 #[no_mangle]
-pub extern fn exp_base64(vector: Box<DataVector64>, value: f64) -> VectorResult<DataVector64> {
-    convert_vec!(vector.exp_base(value))
+pub extern fn expf64(vector: Box<DataVector64>, value: f64) -> VectorResult<DataVector64> {
+    convert_vec!(vector.expf(value))
 }
 
 #[no_mangle]
@@ -341,8 +343,8 @@ pub extern fn magnitude_squared64(vector: Box<DataVector64>) -> VectorResult<Dat
 }
 
 #[no_mangle]
-pub extern fn complex_conj64(vector: Box<DataVector64>) -> VectorResult<DataVector64> {
-    convert_vec!(vector.complex_conj())
+pub extern fn conj64(vector: Box<DataVector64>) -> VectorResult<DataVector64> {
+    convert_vec!(vector.conj())
 }
 
 #[no_mangle]
@@ -355,6 +357,28 @@ pub extern fn to_imag64(vector: Box<DataVector64>) -> VectorResult<DataVector64>
     convert_vec!(vector.to_imag())
 }
 
+#[no_mangle]
+pub extern fn map_inplace_complex64(vector: Box<DataVector64>, map: extern fn(Complex64, usize) -> Complex64) -> VectorResult<DataVector64> {
+    convert_vec!(vector.map_inplace_complex((), move|v, i, _|map(v, i)))
+}
+
+/// Warning: This function interface heavily works around the Rust type system and the safety
+/// it provides. Use with great care!
+#[no_mangle]
+pub extern fn map_aggregate_complex64(vector: &DataVector64, map: extern fn(Complex64, usize) -> *const c_void, aggregate: extern fn(*const c_void, *const c_void) -> *const c_void) -> ScalarResult<*const c_void> {
+    unsafe 
+    {
+        let result = convert_scalar!(
+            vector.map_aggregate_complex(
+                (), 
+                move|v, i, _| mem::transmute(map(v, i)),
+                move|a: usize, b: usize| mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))),
+            mem::transmute(0usize)
+        );
+        mem::transmute(result)
+    }
+}
+
 #[no_mangle]
 pub extern fn get_real64(vector: Box<DataVector64>, destination: &mut DataVector64) -> i32 {
     convert_void!(vector.get_real(destination))
@@ -842,3 +866,524 @@ pub extern fn interpolate_lin64(vector: Box<DataVector64>, interpolation_factor:
 pub extern fn interpolate_hermite64(vector: Box<DataVector64>, interpolation_factor: f64, delay: f64) -> VectorResult<DataVector64> {
     convert_vec!(vector.interpolate_hermite(interpolation_factor, delay))
 }  
+
+pub type PreparedOp1F32 = PreparedOperation1<f64, DataVector64, DataVector64>;
+
+pub type PreparedOp2F32 = PreparedOperation2<f64, DataVector64, DataVector64, DataVector64, DataVector64>;
+
+/// Prepares an operation.
+/// multi_ops1 will not be made available in for interop since the same functionality 
+/// can be created with prepared ops, and internally this is what this lib does too.
+#[no_mangle]
+pub extern fn prepared_ops1_f64() -> Box<PreparedOp1F32> {
+    Box::new(prepare1::<f64, DataVector64>())
+}
+
+/// Prepares an operation.
+/// multi_ops2 will not be made available in for interop since the same functionality 
+/// can be created with prepared ops, and internally this is what this lib does too.
+#[no_mangle]
+pub extern fn prepared_ops2_f64() -> Box<PreparedOp2F32> {
+    Box::new(prepare2::<f64, DataVector64, DataVector64>())
+}
+
+/// Prepares an operation.
+/// multi_ops1 will not be made available in for interop since the same functionality 
+/// can be created with prepared ops, and internally this is what this lib does too.
+#[no_mangle]
+pub extern fn extend_prepared_ops1_f64(ops: Box<PreparedOp1F32>) -> Box<PreparedOp2F32> {
+    Box::new(ops.extend::<DataVector64>())
+}
+
+#[no_mangle]
+pub extern fn exec_prepared_ops1_f64(
+    ops: &PreparedOp1F32,
+    v: Box<DataVector64>) -> VectorResult<DataVector64> {
+    convert_vec!(ops.exec(*v))
+}
+
+#[no_mangle]
+pub extern fn exec_prepared_ops2_f64(
+    ops: &PreparedOp2F32, 
+    v1: Box<DataVector64>, 
+    v2: Box<DataVector64>) -> BinaryVectorResult<DataVector64> {
+    convert_bin_vec!(ops.exec(*v1, *v2))
+}
+
+//----------------------------------------------
+// PreparedOp1F32
+//----------------------------------------------
+#[no_mangle]
+pub extern fn add_real_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::AddReal(arg, value))
+}
+
+#[no_mangle]
+pub extern fn multiply_real_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::MultiplyReal(arg, value))
+}
+
+#[no_mangle]
+pub extern fn abs_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Abs(arg))
+}
+
+#[no_mangle]
+pub extern fn to_complex_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ToComplex(arg))
+}
+
+#[no_mangle]
+pub extern fn map_inplace_real64(vector: Box<DataVector64>, map: extern fn(f64, usize) -> f64) -> VectorResult<DataVector64> {
+    convert_vec!(vector.map_inplace_real((), move|v, i, _|map(v, i)))
+}
+
+/// Warning: This function interface heavily works around the Rust type system and the safety
+/// it provides. Use with great care!
+#[no_mangle]
+pub extern fn map_aggregate_real64(vector: &DataVector64, map: extern fn(f64, usize) -> *const c_void, aggregate: extern fn(*const c_void, *const c_void) -> *const c_void) -> ScalarResult<*const c_void> {
+    unsafe 
+    {
+        let result = convert_scalar!(
+            vector.map_aggregate_real(
+                (), 
+                move|v, i, _| mem::transmute(map(v, i)),
+                move|a: usize, b: usize| mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))),
+            mem::transmute(0usize)
+        );
+        mem::transmute(result)
+    }
+}
+
+#[no_mangle]
+pub extern fn add_complex_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, re: f64, im: f64) {
+    ops.add_enum_op(Operation::AddComplex(arg, Complex64::new(re, im)))
+}
+
+#[no_mangle]
+pub extern fn multiply_complex_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, re: f64, im: f64) {
+    ops.add_enum_op(Operation::MultiplyComplex(arg, Complex64::new(re, im)))
+}
+
+#[no_mangle]
+pub extern fn magnitude_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Magnitude(arg))
+}
+
+#[no_mangle]
+pub extern fn magnitude_squared_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::MagnitudeSquared(arg))
+}
+
+#[no_mangle]
+pub extern fn complex_conj_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ComplexConj(arg))
+}
+
+#[no_mangle]
+pub extern fn to_real_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ToReal(arg))
+}
+
+#[no_mangle]
+pub extern fn to_imag_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ToImag(arg))
+}
+
+#[no_mangle]
+pub extern fn phase_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Phase(arg))
+}
+
+#[no_mangle]
+pub extern fn multiply_complex_exponential_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, a: f64, b: f64) {
+    ops.add_enum_op(Operation::MultiplyComplexExponential(arg, a, b))
+}
+
+#[no_mangle]
+pub extern fn add_vector_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::AddVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn mul_vector_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::MulVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn sub_vector_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::SubVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn div_vector_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::DivVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn square_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Square(arg))
+}
+
+#[no_mangle]
+pub extern fn sqrt_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Sqrt(arg))
+}
+
+#[no_mangle]
+pub extern fn root_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Root(arg, value))
+}
+
+#[no_mangle]
+pub extern fn powf_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Powf(arg, value))
+}
+
+#[no_mangle]
+pub extern fn ln_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Ln(arg))
+}
+
+#[no_mangle]
+pub extern fn exp_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Exp(arg))
+}
+
+#[no_mangle]
+pub extern fn log_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Log(arg, value))
+}
+
+#[no_mangle]
+pub extern fn expf_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Expf(arg, value))
+}
+
+#[no_mangle]
+pub extern fn sin_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Sin(arg))
+}
+
+#[no_mangle]
+pub extern fn cos_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Cos(arg))
+}
+
+#[no_mangle]
+pub extern fn tan_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Tan(arg))
+}
+
+#[no_mangle]
+pub extern fn asin_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ASin(arg))
+}
+
+#[no_mangle]
+pub extern fn acos_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ACos(arg))
+}
+
+#[no_mangle]
+pub extern fn atan_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ATan(arg))
+}
+
+#[no_mangle]
+pub extern fn sinh_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Sinh(arg))
+}
+
+#[no_mangle]
+pub extern fn cosh_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Cosh(arg))
+}
+
+#[no_mangle]
+pub extern fn tanh_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::Tanh(arg))
+}
+
+#[no_mangle]
+pub extern fn asinh_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ASinh(arg))
+}
+
+#[no_mangle]
+pub extern fn acosh_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ACosh(arg))
+}
+
+#[no_mangle]
+pub extern fn atanh_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::ATanh(arg))
+}
+
+#[no_mangle]
+pub extern fn clone_from_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, source: usize) {
+    ops.add_enum_op(Operation::CloneFrom(arg, source))
+}
+
+#[no_mangle]
+pub extern fn add_points_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::AddPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn sub_points_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::SubPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn mul_points_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::MulPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn div_points_ops1_f64(ops: &mut PreparedOp1F32, arg: usize) {
+    ops.add_enum_op(Operation::DivPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn map_real_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, map: extern fn(f64, usize) -> f64) {
+    ops.add_enum_op(Operation::MapReal(arg, Arc::new(move|v, i|map(v, i))))
+}
+
+#[no_mangle]
+pub extern fn map_complex_ops1_f64(ops: &mut PreparedOp1F32, arg: usize, map: extern fn(Complex64, usize) -> Complex64) {
+    ops.add_enum_op(Operation::MapComplex(arg, Arc::new(move|v, i|map(v, i))))
+}
+
+#[no_mangle]
+pub extern fn delete_ops1_f64(vector: Box<PreparedOp1F32>) {
+    drop(vector);
+}
+
+//----------------------------------------------
+// PreparedOp2F32
+//----------------------------------------------
+#[no_mangle]
+pub extern fn add_real_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::AddReal(arg, value))
+}
+
+#[no_mangle]
+pub extern fn multiply_real_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::MultiplyReal(arg, value))
+}
+
+#[no_mangle]
+pub extern fn abs_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Abs(arg))
+}
+
+#[no_mangle]
+pub extern fn to_complex_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ToComplex(arg))
+}
+
+#[no_mangle]
+pub extern fn add_complex_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, re: f64, im: f64) {
+    ops.add_enum_op(Operation::AddComplex(arg, Complex64::new(re, im)))
+}
+
+#[no_mangle]
+pub extern fn multiply_complex_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, re: f64, im: f64) {
+    ops.add_enum_op(Operation::MultiplyComplex(arg, Complex64::new(re, im)))
+}
+
+#[no_mangle]
+pub extern fn magnitude_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Magnitude(arg))
+}
+
+#[no_mangle]
+pub extern fn magnitude_squared_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::MagnitudeSquared(arg))
+}
+
+#[no_mangle]
+pub extern fn complex_conj_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ComplexConj(arg))
+}
+
+#[no_mangle]
+pub extern fn to_real_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ToReal(arg))
+}
+
+#[no_mangle]
+pub extern fn to_imag_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ToImag(arg))
+}
+
+#[no_mangle]
+pub extern fn phase_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Phase(arg))
+}
+
+#[no_mangle]
+pub extern fn multiply_complex_exponential_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, a: f64, b: f64) {
+    ops.add_enum_op(Operation::MultiplyComplexExponential(arg, a, b))
+}
+
+#[no_mangle]
+pub extern fn add_vector_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::AddVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn mul_vector_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::MulVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn sub_vector_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::SubVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn div_vector_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, other: usize) {
+    ops.add_enum_op(Operation::DivVector(arg, other))
+}
+
+#[no_mangle]
+pub extern fn square_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Square(arg))
+}
+
+#[no_mangle]
+pub extern fn sqrt_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Sqrt(arg))
+}
+
+#[no_mangle]
+pub extern fn root_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Root(arg, value))
+}
+
+#[no_mangle]
+pub extern fn powf_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Powf(arg, value))
+}
+
+#[no_mangle]
+pub extern fn ln_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Ln(arg))
+}
+
+#[no_mangle]
+pub extern fn exp_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Exp(arg))
+}
+
+#[no_mangle]
+pub extern fn log_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Log(arg, value))
+}
+
+#[no_mangle]
+pub extern fn expf_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, value: f64) {
+    ops.add_enum_op(Operation::Expf(arg, value))
+}
+
+#[no_mangle]
+pub extern fn sin_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Sin(arg))
+}
+
+#[no_mangle]
+pub extern fn cos_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Cos(arg))
+}
+
+#[no_mangle]
+pub extern fn tan_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Tan(arg))
+}
+
+#[no_mangle]
+pub extern fn asin_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ASin(arg))
+}
+
+#[no_mangle]
+pub extern fn acos_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ACos(arg))
+}
+
+#[no_mangle]
+pub extern fn atan_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ATan(arg))
+}
+
+#[no_mangle]
+pub extern fn sinh_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Sinh(arg))
+}
+
+#[no_mangle]
+pub extern fn cosh_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Cosh(arg))
+}
+
+#[no_mangle]
+pub extern fn tanh_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::Tanh(arg))
+}
+
+#[no_mangle]
+pub extern fn asinh_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ASinh(arg))
+}
+
+#[no_mangle]
+pub extern fn acosh_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ACosh(arg))
+}
+
+#[no_mangle]
+pub extern fn atanh_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::ATanh(arg))
+}
+
+#[no_mangle]
+pub extern fn clone_from_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, source: usize) {
+    ops.add_enum_op(Operation::CloneFrom(arg, source))
+}
+
+#[no_mangle]
+pub extern fn add_points_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::AddPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn sub_points_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::SubPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn mul_points_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::MulPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn div_points_ops2_f64(ops: &mut PreparedOp2F32, arg: usize) {
+    ops.add_enum_op(Operation::DivPoints(arg))
+}
+
+#[no_mangle]
+pub extern fn map_real_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, map: extern fn(f64, usize) -> f64) {
+    ops.add_enum_op(Operation::MapReal(arg, Arc::new(move|v, i|map(v, i))))
+}
+
+#[no_mangle]
+pub extern fn map_complex_ops2_f64(ops: &mut PreparedOp2F32, arg: usize, map: extern fn(Complex64, usize) -> Complex64) {
+    ops.add_enum_op(Operation::MapComplex(arg, Arc::new(move|v, i|map(v, i))))
+}
+
+#[no_mangle]
+pub extern fn delete_ops2_f64(vector: Box<PreparedOp2F32>) {
+    drop(vector);
+}
