@@ -9,10 +9,9 @@ macro_rules! add_basic_private_impl {
                         F: Fn($data_type, A) -> $data_type + 'static + Sync {
                     {
                         let mut array = &mut self.data;
-                        let length = array.len();
                         Chunk::execute_partial(
                             complexity, &self.multicore_settings,
-                            &mut array, length, 1, argument,
+                            &mut array, 1, argument,
                             move|array, argument| {
                                 for num in array {
                                     *num = op(*num, argument);
@@ -29,19 +28,24 @@ macro_rules! add_basic_private_impl {
                             G: Fn($data_type, A) -> $data_type + 'static + Sync {
                     {
                         let data_length = self.len();
-                        let scalar_length = data_length % $reg::len();
-                        let vectorization_length = data_length - scalar_length;
+                        let (scalar_left, scalar_right, vectorization_length) = $reg::calc_data_alignment_reqs(&self.data[0..data_length]);
                         let mut array = &mut self.data;
-                        Chunk::execute_partial(
-                            complexity, &self.multicore_settings,
-                            &mut array, vectorization_length, $reg::len(), argument, 
-                            move |array, argument| {
+                        if vectorization_length > 0 {
+                            Chunk::execute_partial(
+                                complexity, &self.multicore_settings,
+                                &mut array[scalar_left..vectorization_length], $reg::len(), argument, 
+                                move |array, argument| {
                                 let array = $reg::array_to_regs_mut(array);
                                 for reg in array {
                                     *reg = simd_op(*reg, argument);
                                 }
-                        });
-                        for num in &mut array[vectorization_length..data_length]
+                            });
+                        }
+                        for num in &mut array[0..scalar_left]
+                        {
+                            *num = scalar_op(*num, argument);
+                        }
+                        for num in &mut array[scalar_right..data_length]
                         {
                             *num = scalar_op(*num, argument);
                         }
@@ -54,11 +58,10 @@ macro_rules! add_basic_private_impl {
                     where A: Sync + Copy + Send,
                         F: Fn(Complex<$data_type>, A) -> Complex<$data_type> + 'static + Sync {
                     {
-                        let length = self.len();
                         let mut array = &mut self.data;
                         Chunk::execute_partial(
                             complexity, &self.multicore_settings,
-                            &mut array, length, 2, argument,
+                            array, 2, argument,
                             move|array, argument| {
                                 let array = Self::array_to_complex_mut(array);
                                 for num in array {
@@ -74,13 +77,13 @@ macro_rules! add_basic_private_impl {
                     where A: Sync + Copy + Send,
                         F: Fn(Complex<$data_type>, A) -> $data_type + 'static + Sync {
                     {
-                        let data_length = self.len();       
+                        let data_length = self.len();   
                         let mut array = &mut self.data;
                         let mut temp = temp_mut!(self, data_length / 2);
                         Chunk::from_src_to_dest(
                             complexity, &self.multicore_settings,
-                            &mut array, data_length, 2, 
-                            &mut temp, data_length / 2, 1, argument,
+                            &mut array[0..data_length], 2, 
+                            &mut temp[0..data_length / 2], 1, argument,
                             move |array, range, target, argument| {
                                 let array = Self::array_to_complex(&array[range.start..range.end]);
                                 for pair in array.iter().zip(target) {
@@ -101,21 +104,32 @@ macro_rules! add_basic_private_impl {
                             G: Fn(Complex<$data_type>, A) -> Complex<$data_type> + 'static + Sync {
                     {
                         let data_length = self.len();
-                        let scalar_length = data_length % $reg::len();
-                        let vectorization_length = data_length - scalar_length;           
+                        let (scalar_left, scalar_right, vectorization_length) = $reg::calc_data_alignment_reqs(&self.data[0..data_length]);
                         let mut array = &mut self.data;
-                        Chunk::execute_partial(
-                            complexity, &self.multicore_settings,
-                            &mut array, vectorization_length, $reg::len(), argument, 
-                            move |array, argument| {
+                        if vectorization_length > 0 {
+                            Chunk::execute_partial(
+                                complexity, &self.multicore_settings,
+                                &mut array[scalar_left..vectorization_length], $reg::len(), argument, 
+                                move |array, argument| {
                                 let array = $reg::array_to_regs_mut(array);
                                 for reg in array {
                                     *reg = simd_op(*reg, argument);
                                 }
-                        });
-                        let array = Self::array_to_complex_mut(&mut array[vectorization_length..data_length]);
-                        for num in array {
-                            *num = scalar_op(*num, argument);
+                            });
+                        }
+                        {
+                            let array = Self::array_to_complex_mut(&mut array[0..scalar_left]);
+                            for num in array
+                            {
+                                *num = scalar_op(*num, argument);
+                            }
+                        }
+                        {
+                            let array = Self::array_to_complex_mut(&mut array[scalar_right..data_length]);
+                            for num in array
+                            {
+                                *num = scalar_op(*num, argument);
+                            }
                         }
                     }
                     Ok(self)
@@ -128,28 +142,39 @@ macro_rules! add_basic_private_impl {
                           G: Fn(Complex<$data_type>, A) -> $data_type + 'static + Sync {
                     {
                         let data_length = self.len();
-                        let scalar_length = data_length % $reg::len();
-                        let vectorization_length = data_length - scalar_length;           
+                        let (scalar_left, scalar_right, vectorization_length) = $reg::calc_data_alignment_reqs(&self.data[0..data_length]);
                         let mut array = &mut self.data;
                         let mut temp = temp_mut!(self, data_length);
-                        Chunk::from_src_to_dest(
+                        if vectorization_length > 0 {
+                            Chunk::from_src_to_dest(
                             complexity, &self.multicore_settings,
-                            &mut array, vectorization_length, $reg::len(), 
-                            &mut temp, vectorization_length / 2, $reg::len() / 2, argument,
+                            &mut array[scalar_left..vectorization_length], $reg::len(), 
+                            &mut temp[scalar_left/2..vectorization_length/2], $reg::len() / 2, argument,
                             move |array, range, target, argument| {
                                 let array = $reg::array_to_regs(&array[range.start..range.end]);
                                 let mut j = 0;
                                 for reg in array {
                                     let result = simd_op(*reg, argument);
-                                    result.store_half(target, j);
+                                    result.store_half_unchecked(target, j);
                                     j += $reg::len() / 2;
                                 }
-                        });
-                        let array = Self::array_to_complex(&array[vectorization_length..data_length]);
-                        for pair in array.iter().zip(&mut temp[vectorization_length/2..data_length/2]) {
-                            let (src, dest) = pair;
-                            *dest = scalar_op(*src, argument);
+                            });
                         }
+                        {
+                            let array = Self::array_to_complex(&array[0..scalar_left]);
+                            for pair in array.iter().zip(&mut temp[0..scalar_left/2]) {
+                                let (src, dest) = pair;
+                                *dest = scalar_op(*src, argument);
+                            }
+                        }
+                        {
+                            let array = Self::array_to_complex(&array[scalar_right..data_length]);
+                            for pair in array.iter().zip(&mut temp[scalar_right/2..data_length/2]) {
+                                let (src, dest) = pair;
+                                *dest = scalar_op(*src, argument);
+                            }
+                        }
+                        
                         self.is_complex = false;
                         self.valid_len = data_length / 2;
                     }
@@ -214,7 +239,7 @@ macro_rules! add_basic_private_impl {
                             let converted = convert_mut(&mut self.data[0..len]);
                             Chunk::execute_with_range(
                                 Complexity::Medium, &self.multicore_settings,
-                                converted, points, 1, function_arg,
+                                converted, 1, function_arg,
                                 move |array, range, arg| {
                                     let scale = T::from(ratio);
                                     let offset = if points % 2 != 0 { 1 } else { 0 };
@@ -234,7 +259,7 @@ macro_rules! add_basic_private_impl {
                             let converted = convert_mut(&mut self.data[0..len]);
                             Chunk::execute_sym_pairs_with_range(
                                 Complexity::Medium, &self.multicore_settings,
-                                converted, points, 1, function_arg,
+                                converted, 1, function_arg,
                                 move |array1, range1, array2, range2, arg| {
                                     assert!(array1.len() >= array2.len());
                                     assert!(range1.end <= range2.start);let scale = T::from(ratio);
@@ -310,7 +335,7 @@ macro_rules! add_basic_private_impl {
                             let converted = convert_mut(&mut self.data[0..len]);
                             Chunk::execute_with_range(
                                 Complexity::Medium, &self.multicore_settings,
-                                converted, points, 1, function_arg,
+                                converted, 1, function_arg,
                                 move |array, range, arg| {
                                     let mut j = range.start;
                                     for num in array {
@@ -327,7 +352,7 @@ macro_rules! add_basic_private_impl {
                             let converted = convert_mut(&mut self.data[0..len]);
                             Chunk::execute_sym_pairs_with_range(
                                 Complexity::Medium, &self.multicore_settings,
-                                converted, points, 1, function_arg,
+                                converted, 1, function_arg,
                                 move |array1, range, array2, _, arg| {
                                     assert!(array1.len() >= array2.len());
                                     let mut j = range.start;
@@ -369,8 +394,8 @@ macro_rules! add_basic_private_impl {
                 }
                 
                 /// Creates shifted and reversed copies of the given data vector. 
-                /// This function is especially desgined for convolutions.
-                fn create_shifted_copies(vector: &GenericDataVector<$data_type>) -> Vec<Vec<$data_type>>{
+                /// This function is especially designed for convolutions.
+                fn create_shifted_copies(vector: &GenericDataVector<$data_type>) -> Vec<Vec<$reg>>{
                     let step = if vector.is_complex { 2 } else { 1 };
                     let number_of_shifts = $reg::len() / step;
                     let mut shifted_copies = Vec::with_capacity(number_of_shifts);
@@ -391,30 +416,58 @@ macro_rules! add_basic_private_impl {
                             x => (number_of_shifts - x) * step
                         };
                         let min_len = vector.len() + shift;
-                        let len =  (min_len + $reg::len() - 1) / $reg::len() * $reg::len();
-                        let mut copy = Vec::with_capacity(len);
+                        let len =  (min_len + $reg::len() - 1) / $reg::len();
+                        let mut copy: Vec<$reg> = Vec::with_capacity(len);
                         
-                        let mut j = len;
+                        let mut j = len  * $reg::len();;
+                        let mut k = 0;
+                        let mut current = $reg::splat(0.0).to_array();
                         while j > 0 {
                             j -= step;
                             if j < shift || j >= min_len {
-                                copy.push(0.0);
+                                current[k] = 0.0;
+                                k += 1;
+                                if k >= current.len() {
+                                    copy.push($reg::load_unchecked(&current, 0));
+                                    k = 0;
+                                }
                                 if step > 1 {
-                                    copy.push(0.0);
+                                    current[k] = 0.0;
+                                    k += 1;
+                                    if k >= current.len() {
+                                        copy.push($reg::load_unchecked(&current, 0));
+                                        k = 0;
+                                    }
                                 }
                             } else {
                                 if step > 1 {
                                     let im = *data.next().unwrap();
                                     let re = *data.next().unwrap();
-                                    copy.push(re);
-                                    copy.push(im);
+                                    current[k] = re;
+                                    k += 1;
+                                    if k >= current.len() {
+                                        copy.push($reg::load_unchecked(&current, 0));
+                                        k = 0;
+                                    }
+                                    current[k] = im;
+                                    k += 1;
+                                    if k >= current.len() {
+                                        copy.push($reg::load_unchecked(&current, 0));
+                                        k = 0;
+                                    }
                                 }
                                 else {
-                                    copy.push(*data.next().unwrap());
+                                    current[k] = *data.next().unwrap();
+                                    k += 1;
+                                    if k >= current.len() {
+                                        copy.push($reg::load_unchecked(&current, 0));
+                                        k = 0;
+                                    }
                                 }
                             }
                         }
                         
+                        assert_eq!(k, 0);
                         assert_eq!(copy.len(), len);
                         shifted_copies.push(copy);
                         i += 1;
