@@ -1,25 +1,29 @@
 use multicore_support::{Chunk, Complexity};
 use super::definitions::{
-	DataVector,
+    DataVector,
     VecResult,
     VoidResult,
     ErrorReason,
     ScalarResult,
     Statistics,
-    Scale,
-    Offset,
-	ComplexVectorOperations};
+    ScaleOps,
+    VectorIter,
+    OffsetOps,
+    DotProductOps,
+    StatisticsOps,
+    ComplexVectorOps};
 use super::GenericDataVector;
 use super::stats_impl::Stats;
-use simd_extensions::{Simd,Reg32,Reg64};
+use simd_extensions::*;
 use num::complex::Complex;
+use std::sync::Arc;
 
 macro_rules! add_complex_impl {
     ($($data_type:ident, $reg:ident);*)
-	 =>
-	 {	 
+     =>
+     {     
         $(
-            impl ComplexVectorOperations<$data_type> for GenericDataVector<$data_type>
+            impl ComplexVectorOps<$data_type> for GenericDataVector<$data_type>
             {
                 type RealPartner = GenericDataVector<$data_type>;
                 
@@ -46,11 +50,31 @@ macro_rules! add_complex_impl {
                     assert_complex!(self);
                     {
                         let a = a * self.delta();
-                        let length = self.len();
-                        let array = &mut self.data;
-                        let array = Self::array_to_complex_mut(&mut array[0..length]);
-                        let mut exponential = Complex::<$data_type>::from_polar(&1.0, &b);
+                        let data_length = self.len();
+                        let scalar_length = data_length % $reg::len();
+                        let vectorization_length = data_length - scalar_length;
+                        let mut array = &mut self.data;
+                        Chunk::execute_with_range(
+                            Complexity::Small, &self.multicore_settings,
+                            &mut array[0..vectorization_length], $reg::len(), 
+                            (a, b),
+                            move |array, range, args| {
+                            let (a, b) = args;
+                            let mut exponential = 
+                                Complex::<$data_type>::from_polar(&1.0, &b) 
+                                * Complex::<$data_type>::from_polar(&1.0, &(a * range.start as $data_type / 2.0));
+                            let increment = Complex::<$data_type>::from_polar(&1.0, &a);
+                            let array = Self::array_to_complex_mut(array);
+                            for complex in array {
+                                *complex = (*complex) * exponential;
+                                exponential = exponential * increment;
+                            }
+                        });
+                        let mut exponential = 
+                            Complex::<$data_type>::from_polar(&1.0, &b) 
+                            * Complex::<$data_type>::from_polar(&1.0, &(a * vectorization_length as $data_type / 2.0));
                         let increment = Complex::<$data_type>::from_polar(&1.0, &a);
+                        let array = Self::array_to_complex_mut(&mut array[vectorization_length..data_length]);
                         for complex in array {
                             *complex = (*complex) * exponential;
                             exponential = exponential * increment;
@@ -80,16 +104,16 @@ macro_rules! add_complex_impl {
                     let mut temp = &mut destination.data;
                     Chunk::from_src_to_dest(
                         Complexity::Small, &self.multicore_settings,
-                        &array, vectorization_length, $reg::len(), 
-                        &mut temp, vectorization_length / 2, $reg::len() / 2, (),
+                        &array[0..vectorization_length], $reg::len(), 
+                        &mut temp[0..vectorization_length/2], $reg::len() / 2, (),
                         move |array, range, target, _arg| {
                             let mut i = 0;
                             let mut j = range.start;
                             while i < target.len()
                             { 
-                                let vector = $reg::load(array, j);
+                                let vector = $reg::load_unchecked(array, j);
                                 let result = vector.complex_abs();
-                                result.store_half(target, i);
+                                result.store_half_unchecked(target, i);
                                 j += $reg::len();
                                 i += $reg::len() / 2;
                             }
@@ -112,54 +136,11 @@ macro_rules! add_complex_impl {
                     self.simd_complex_to_real_operation(|x,_arg| x.complex_abs_squared(), |x,_arg| x.re * x.re + x.im * x.im, (), Complexity::Small)
                 }
                 
-                fn complex_conj(mut self) -> VecResult<Self>
+                fn conj(self) -> VecResult<Self>
                 {
                     assert_complex!(self);
-                    {
-                        let data_length = self.len();
-                        if data_length == 0 {
-                            return Ok(self);
-                        }
-                        
-                        let unroll_len = 8;
-                        let scalar_length = data_length % unroll_len;
-                        let unrolled_length = data_length - scalar_length;           
-                        let mut array = &mut self.data;
-                        Chunk::execute_partial(
-                            Complexity::Small, &self.multicore_settings,
-                            &mut array, unrolled_length, unroll_len, (), 
-                            move |array, _| {
-                                unsafe {
-                                    let array_len = array.len();
-                                    if array_len == 0 {
-                                        return;
-                                    }
-                                    let end = &mut array[array_len - 1] as *mut $data_type;
-                                    let mut array = &mut array[1] as *mut $data_type;
-                                    while array <= end {
-                                        *array = -(*array);
-                                        array = array.offset(2);
-                                        *array = -(*array);
-                                        array = array.offset(2);
-                                        *array = -(*array);
-                                        array = array.offset(2);
-                                        *array = -(*array);
-                                        array = array.offset(2);
-                                    }
-                                }
-                            });
-                        if data_length > unrolled_length {
-                            let end = &mut array[data_length - 1] as *mut $data_type;
-                            let mut array = &mut array[unrolled_length + 1] as *mut $data_type;
-                            unsafe {
-                                while array <= end {
-                                    *array = -(*array);
-                                    array = array.offset(2);
-                                }
-                            }
-                        }
-                    }
-                    Ok(self)
+                    let factor = $reg::from_complex(Complex::<$data_type>::new(1.0, -1.0));
+                    self.simd_complex_operation(|x,y| x * y, |x,_| x.conj(), factor, Complexity::Small)
                 }
                 
                 fn to_real(self) -> VecResult<Self>
@@ -172,7 +153,7 @@ macro_rules! add_complex_impl {
                 {
                     assert_complex!(self);
                     self.pure_complex_to_real_operation(|x,_arg|x.im, (), Complexity::Small)
-                }	
+                }    
                         
                 fn get_real(&self, destination: &mut Self) -> VoidResult
                 {
@@ -207,6 +188,91 @@ macro_rules! add_complex_impl {
                     self.pure_complex_into_real_target_operation(destination, |x,_arg|x.arg(), (), Complexity::Small)
                 }
                 
+                fn map_inplace_complex<A, F>(mut self, argument: A, f: F) -> VecResult<Self>
+                    where A: Sync + Copy + Send,
+                          F: Fn(Complex<$data_type>, usize, A) -> Complex<$data_type> + 'static + Sync {
+                    {
+                        assert_complex!(self);
+                        let mut array = &mut self.data;
+                        let length = array.len();
+                        Chunk::execute_with_range(
+                            Complexity::Small, &self.multicore_settings,
+                            &mut array[0..length], 2, argument,
+                            move|array, range, argument| {
+                                let mut i = range.start / 2;
+                                let array = Self::array_to_complex_mut(array);
+                                for num in array {
+                                    *num = f(*num, i, argument);
+                                    i += 1;
+                                }
+                            });
+                    }
+                    Ok(self)
+                }
+                
+                fn map_aggregate_complex<A, FMap, FAggr, R>(
+                    &self, 
+                    argument: A, 
+                    map: FMap,
+                    aggregate: FAggr) -> ScalarResult<R>
+                        where A: Sync + Copy + Send,
+                              FMap: Fn(Complex<$data_type>, usize, A) -> R + 'static + Sync,
+                              FAggr: Fn(R, R) -> R + 'static + Sync + Send,
+                              R: Send {
+                    
+                    let aggregate = Arc::new(aggregate);
+                    let mut result = {
+                        if !self.is_complex {
+                            return Err(ErrorReason::VectorMustBeComplex);
+                        }
+                        
+                        let array = &self.data;
+                        let length = array.len();
+                        if length == 0 {
+                            return Err(ErrorReason::VectorMustNotBeEmpty);
+                        }
+                        let aggregate  = aggregate.clone();
+                        Chunk::map_on_array_chunks(
+                            Complexity::Small, &self.multicore_settings,
+                            &array[0..length], 2, argument,
+                            move|array, range, argument| {
+                                let aggregate  = aggregate.clone();
+                                let array = Self::array_to_complex(array);
+                                let mut i = range.start / 2;
+                                let mut sum: Option<R> = None;
+                                for num in array {
+                                    let res = map(*num, i, argument);
+                                    sum = match sum {
+                                        None => Some(res),
+                                        Some(s) => Some(aggregate(s, res))
+                                    };
+                                    i += 1;
+                                }
+                                sum
+                            })
+                    };
+                    let aggregate  = aggregate.clone();
+                    // Would be nicer if we could use iter().fold(..) but we need
+                    // the value of R and not just a reference so we can't user an iter
+                    let mut only_valid_options = Vec::with_capacity(result.len());
+                    for _ in 0..result.len() {
+                        let elem = result.pop().unwrap();
+                        match elem {
+                            None => (),
+                            Some(e) => only_valid_options.push(e)
+                        };
+                    }
+                    
+                    if only_valid_options.len() == 0 {
+                        return Err(ErrorReason::VectorMustNotBeEmpty);
+                    }
+                    let mut aggregated = only_valid_options.pop().unwrap();
+                    for _ in 0..only_valid_options.len() {
+                        aggregated = aggregate(aggregated, only_valid_options.pop().unwrap());
+                    }
+                    Ok(aggregated)
+                }
+                
                 fn complex_dot_product(&self, factor: &Self) -> ScalarResult<Complex<$data_type>>
                 {
                     if !self.is_complex {
@@ -225,16 +291,16 @@ macro_rules! add_complex_impl {
                     let other = &factor.data;
                     let chunks = Chunk::get_a_fold_b(
                         Complexity::Small, &self.multicore_settings,
-                        &other, vectorization_length, $reg::len(), 
-                        &array, vectorization_length, $reg::len(), 
+                        &other[0..vectorization_length], $reg::len(), 
+                        &array[0..vectorization_length], $reg::len(), 
                         |original, range, target| {
                             let mut i = 0;
                             let mut j = range.start;
                             let mut result = $reg::splat(0.0);
                             while i < target.len()
                             { 
-                                let vector1 = $reg::load(original, j);
-                                let vector2 = $reg::load(target, i);
+                                let vector1 = $reg::load_unchecked(original, j);
+                                let vector2 = $reg::load_unchecked(target, i);
                                 result = result + (vector2.mul_complex(vector1));
                                 i += $reg::len();
                                 j += $reg::len();
@@ -261,7 +327,7 @@ macro_rules! add_complex_impl {
                     let array = &self.data;
                     let chunks = Chunk::get_chunked_results(
                         Complexity::Small, &self.multicore_settings,
-                        &array, data_length, 2, (),
+                        &array[0..data_length], 2, (),
                         |array, range, _arg| {
                             let mut stat = Statistics::<Complex<$data_type>>::empty();
                             let mut i = 0;
@@ -288,7 +354,7 @@ macro_rules! add_complex_impl {
                     let array = &self.data;
                     let chunks = Chunk::get_chunked_results (
                         Complexity::Small, &self.multicore_settings,
-                        &array, data_length, 2, len,
+                        &array[0..data_length], 2, len,
                         |array, range, len| {
                             let mut results = Statistics::<Complex<$data_type>>::empty_vec(len);
                             let mut i = 0;
@@ -390,8 +456,8 @@ macro_rules! add_complex_impl {
                     let source = &self.data;
                     Chunk::from_src_to_dest(
                         complexity, &self.multicore_settings,
-                        &source, len, 2, 
-                        &mut array, len / 2, 1, argument,
+                        &source[0..len], 2, 
+                        &mut array[0..len/2], 1, argument,
                         move|original, range, target, argument| {
                             let mut i = range.start;
                             let mut j = 0;
@@ -407,17 +473,53 @@ macro_rules! add_complex_impl {
                 }
             }
             
-            impl Scale<Complex<$data_type>> for GenericDataVector<$data_type> {
+            impl ScaleOps<Complex<$data_type>> for GenericDataVector<$data_type> {
                 fn scale(self, offset: Complex<$data_type>) -> VecResult<Self> {
                     self.complex_scale(offset)
                 }
             }
             
-            impl Offset<Complex<$data_type>> for GenericDataVector<$data_type> {
+            impl OffsetOps<Complex<$data_type>> for GenericDataVector<$data_type> {
                 fn offset(self, offset: Complex<$data_type>) -> VecResult<Self> {
                     self.complex_offset(offset)
                 }
             }
+            
+            impl DotProductOps<Complex<$data_type>> for GenericDataVector<$data_type> {
+                fn dot_product(&self, factor: &Self) -> ScalarResult<Complex<$data_type>> {
+                    self.complex_dot_product(factor)
+                }
+            }
+            
+            impl StatisticsOps<Complex<$data_type>> for GenericDataVector<$data_type> {
+                fn statistics(&self) -> Statistics<Complex<$data_type>> {
+                    self.complex_statistics()
+                }
+                
+                fn statistics_splitted(&self, len: usize) -> Vec<Statistics<Complex<$data_type>>> {
+                    self.complex_statistics_splitted(len)
+                }
+            }
+            
+            impl VectorIter<Complex<$data_type>> for GenericDataVector<$data_type> {
+                fn map_inplace<A, F>(self, argument: A, map: F) -> VecResult<Self>
+                    where A: Sync + Copy + Send,
+                          F: Fn(Complex<$data_type>, usize, A) -> Complex<$data_type> + 'static + Sync {
+                    self.map_inplace_complex(argument, map)
+                }
+                
+                fn map_aggregate<A, FMap, FAggr, R>(
+                    &self, 
+                    argument: A, 
+                    map: FMap,
+                    aggregate: FAggr) -> ScalarResult<R>
+                where A: Sync + Copy + Send,
+                      FMap: Fn(Complex<$data_type>, usize, A) -> R + 'static + Sync,
+                      FAggr: Fn(R, R) -> R + 'static + Sync + Send,
+                      R: Send {
+                    self.map_aggregate_complex(argument, map, aggregate)  
+                }
+            }
         )*
      }
 }
