@@ -1,8 +1,8 @@
 use multicore_support::{Chunk, Complexity};
 use super::definitions::{
-	DataVector,
+    DataVector,
     DataVectorDomain,
-	GenericVectorOperations,
+    GenericVectorOps,
     VecResult,
     VoidResult,
     ErrorReason,
@@ -11,8 +11,7 @@ use super::{
     GenericDataVector,
     round_len};
 use num::complex::Complex;
-use complex_extensions::ComplexExtensions;
-use simd_extensions::{Simd, Reg32, Reg64};
+use simd_extensions::*;
 use multicore_support::MultiCoreSettings;
 use std::ops::{Add, Sub, Mul, Div};
 use std::ptr;
@@ -91,17 +90,17 @@ macro_rules! impl_binary_vector_operation {
                 let other = &$arg_name.data;
                 Chunk::from_src_to_dest(
                     Complexity::Small, &self.multicore_settings,
-                    &other, vectorization_length, $reg::len(), 
-                    &mut array, vectorization_length, $reg::len(), (), 
+                    &other[0..vectorization_length], $reg::len(), 
+                    &mut array[0..vectorization_length], $reg::len(), (), 
                     |original, range, target, _arg| {
                         let mut i = 0;
                         let mut j = range.start;
                         while i < target.len()
                         { 
-                            let vector1 = $reg::load(original, j);
-                            let vector2 = $reg::load(target, i);
+                            let vector1 = $reg::load_unchecked(original, j);
+                            let vector2 = $reg::load_unchecked(target, i);
                             let result = vector2.$simd_op(vector1);
-                            result.store(target, i);
+                            result.store_unchecked(target, i);
                             i += $reg::len();
                             j += $reg::len();
                         }
@@ -135,17 +134,17 @@ macro_rules! impl_binary_complex_vector_operation {
                 let other = &$arg_name.data;
                 Chunk::from_src_to_dest(
                     Complexity::Small, &self.multicore_settings,
-                    &other, vectorization_length, $reg::len(), 
-                    &mut array, vectorization_length, $reg::len(), (),
+                    &other[0..vectorization_length], $reg::len(), 
+                    &mut array[0..vectorization_length], $reg::len(), (),
                     |original, range, target, _arg| {
                         let mut i = 0;
                         let mut j = range.start;
                         while i < target.len()
                         { 
-                            let vector1 = $reg::load(original, j);
-                            let vector2 = $reg::load(target, i);
+                            let vector1 = $reg::load_unchecked(original, j);
+                            let vector2 = $reg::load_unchecked(target, i);
                             let result = vector2.$simd_op(vector1);
-                            result.store(target, i);
+                            result.store_unchecked(target, i);
                             i += $reg::len();
                             j += $reg::len();
                         }
@@ -183,8 +182,8 @@ macro_rules! impl_binary_smaller_vector_operation {
                 let other = &$arg_name.data;
                 Chunk::from_src_to_dest(
                     Complexity::Small, &self.multicore_settings,
-                    &other, vectorization_length, $reg::len(), 
-                    &mut array, vectorization_length, $reg::len(), (),
+                    &other, $reg::len(), 
+                    &mut array[0..vectorization_length], $reg::len(), (),
                     |original, range, target, _arg| {
                         // This parallelization likely doesn't make sense for the use 
                         // case which we have in mind with this implementation
@@ -195,13 +194,13 @@ macro_rules! impl_binary_smaller_vector_operation {
                         { 
                             let vector1 = 
                                 if j + $reg::len() < original.len() { 
-                                    $reg::load(original, j)
+                                    $reg::load_unchecked(original, j)
                                 } else {
-                                    $reg::load_wrap(original, j)
+                                    $reg::load_wrap_unchecked(original, j)
                                 };
-                            let vector2 = $reg::load(target, i);
+                            let vector2 = $reg::load_unchecked(target, i);
                             let result = vector2.$simd_op(vector1);
-                            result.store(target, i);
+                            result.store_unchecked(target, i);
                             i += $reg::len();
                             j = (j + $reg::len()) % original.len();
                         }
@@ -235,8 +234,8 @@ macro_rules! impl_binary_smaller_complex_vector_operation {
                 let other = &$arg_name.data;
                 Chunk::from_src_to_dest(
                     Complexity::Small, &self.multicore_settings,
-                    &other, vectorization_length, $reg::len(), 
-                    &mut array, vectorization_length, $reg::len(), (),
+                    &other, $reg::len(), 
+                    &mut array[0..vectorization_length], $reg::len(), (),
                     |original, range, target, _arg| {
                         // This parallelization likely doesn't make sense for the use 
                         // case which we have in mind with this implementation
@@ -247,13 +246,13 @@ macro_rules! impl_binary_smaller_complex_vector_operation {
                         { 
                             let vector1 = 
                                 if j + $reg::len() < original.len() { 
-                                    $reg::load(original, j)
+                                    $reg::load_unchecked(original, j)
                                 } else {
-                                    $reg::load_wrap(original, j)
+                                    $reg::load_wrap_unchecked(original, j)
                                 };
-                            let vector2 = $reg::load(target, i);
+                            let vector2 = $reg::load_unchecked(target, i);
                             let result = vector2.$simd_op(vector1);
-                            result.store(target, i);
+                            result.store_unchecked(target, i);
                             i += $reg::len();
                             j = (j + $reg::len()) % original.len();
                         }
@@ -282,8 +281,8 @@ macro_rules! vector_diff_par {
         }
         Chunk::from_src_to_dest(
             Complexity::Small, &$self_.multicore_settings,
-            &$org, $data_length, $step, 
-            &mut $target, $data_length, $step, (),
+            &$org[0..$data_length], $step, 
+            &mut $target[0..$data_length], $step, (),
             |original, range, target, _arg| {
                 let mut i = 0;
                 let mut j = range.start;
@@ -346,8 +345,8 @@ macro_rules! zero_interleave {
                 let source = &$self_.data;
                 Chunk::from_src_to_dest(
                     Complexity::Small, &$self_.multicore_settings,
-                    &source, old_len, $tuple, 
-                    &mut target, new_len, $tuple * step, (),
+                    &source[0..old_len], $tuple, 
+                    &mut target[0..new_len], $tuple * step, (),
                     move|original, range, target, _arg| {
                          // Zero target
                         let ptr = &mut target[0] as *mut $data_type;
@@ -376,8 +375,8 @@ macro_rules! zero_interleave {
 
 macro_rules! add_general_impl {
     ($($data_type:ident, $reg:ident);*)
-	 =>
-	 {	 
+     =>
+     {     
         $(
             impl GenericDataVector<$data_type> {
                 /// Same as `new` but also allows to set multicore options.
@@ -480,7 +479,7 @@ macro_rules! add_general_impl {
                 }
             }
             
-            impl GenericVectorOperations<$data_type> for GenericDataVector<$data_type> {
+            impl GenericVectorOps<$data_type> for GenericDataVector<$data_type> {
                 impl_binary_vector_operation!($data_type, $reg, fn add_vector, summand, add, add);
                 impl_binary_smaller_vector_operation!($data_type, $reg, fn add_smaller_vector, summand, add, add);
                 impl_binary_vector_operation!($data_type, $reg, fn subtract_vector, summand, sub, sub);
@@ -679,11 +678,11 @@ macro_rules! add_general_impl {
                 impl_real_complex_dispatch!(fn sqrt, real_sqrt, complex_sqrt);
                 impl_real_complex_dispatch!(fn square, real_square, complex_square);
                 impl_real_complex_arg_dispatch!(fn root, $data_type, degree, real_root, complex_root);
-                impl_real_complex_arg_dispatch!(fn power, $data_type, exponent, real_power, complex_power);
-                impl_real_complex_dispatch!(fn logn, real_logn, complex_logn);
-                impl_real_complex_dispatch!(fn expn, real_expn, complex_expn);
-                impl_real_complex_arg_dispatch!(fn log_base, $data_type, base, real_log_base, complex_log_base);
-                impl_real_complex_arg_dispatch!(fn exp_base, $data_type, base, real_exp_base, complex_exp_base);
+                impl_real_complex_arg_dispatch!(fn powf, $data_type, exponent, real_powf, complex_powf);
+                impl_real_complex_dispatch!(fn ln, real_ln, complex_ln);
+                impl_real_complex_dispatch!(fn exp, real_exp, complex_exp);
+                impl_real_complex_arg_dispatch!(fn log, $data_type, base, real_log, complex_log);
+                impl_real_complex_arg_dispatch!(fn expf, $data_type, base, real_expf, complex_expf);
                 impl_real_complex_dispatch!(fn sin, real_sin, complex_sin);
                 impl_real_complex_dispatch!(fn cos, real_cos, complex_cos);
                 impl_real_complex_dispatch!(fn tan, real_tan, complex_tan);
@@ -827,19 +826,19 @@ macro_rules! add_general_impl {
                     self.pure_complex_operation(|x,y|x.powf(1.0 / y), base, Complexity::Medium)
                 }
                 
-                impl_function_call_real_arg_complex!($data_type; fn real_power, powf; fn complex_power, powf);
-                impl_function_call_real_complex!($data_type; fn real_logn, ln; fn complex_logn, ln);
-                impl_function_call_real_complex!($data_type; fn real_expn, exp; fn complex_expn, expn);
-                impl_function_call_real_arg_complex!($data_type; fn real_log_base, log; fn complex_log_base, log_base);
+                impl_function_call_real_arg_complex!($data_type; fn real_powf, powf; fn complex_powf, powf);
+                impl_function_call_real_complex!($data_type; fn real_ln, ln; fn complex_ln, ln);
+                impl_function_call_real_complex!($data_type; fn real_exp, exp; fn complex_exp, exp);
+                impl_function_call_real_arg_complex!($data_type; fn real_log, log; fn complex_log, log);
                 
-                fn real_exp_base(self, base: $data_type) -> VecResult<Self>
+                fn real_expf(self, base: $data_type) -> VecResult<Self>
                 {
                     self.pure_real_operation(|x,y|y.powf(x), base, Complexity::Medium)
                 }
                 
-                fn complex_exp_base(self, base: $data_type) -> VecResult<Self>
+                fn complex_expf(self, base: $data_type) -> VecResult<Self>
                 {
-                    self.pure_complex_operation(|x,y|x.exp_base(y), base, Complexity::Medium)
+                    self.pure_complex_operation(|x,y|x.expf(y), base, Complexity::Medium)
                 }
                                
                 impl_function_call_real_complex!($data_type; fn real_sin, sin; fn complex_sin, sin);
