@@ -1,10 +1,10 @@
 use super::definitions::{
-	DataVector,
+    DataVector,
     VecResult,
     DataVectorDomain,
     ErrorReason,
-    ComplexVectorOperations,
-    RealVectorOperations};
+    ComplexVectorOps,
+    RealVectorOps};
 use super::{
     GenericDataVector,
     RealTimeVector,
@@ -23,49 +23,49 @@ use std::ops::{Mul, Div};
 /// All operations in this trait fail with `VectorMustBeInTimeDomain` if the vector isn't in time domain.
 pub trait TimeDomainOperations<T> : DataVector<T> 
     where T : RealNumber {
-	type FreqPartner;
-	
-	/// Performs a Fast Fourier Transformation transforming a time domain vector
-	/// into a frequency domain vector. 
-	/// 
-	/// This version of the FFT neither applies a window nor does it scale the 
-	/// vector.
-	/// # Example
-	///
-	/// ```
-	/// use basic_dsp::{ComplexTimeVector32, TimeDomainOperations, DataVector};
-	/// let vector = ComplexTimeVector32::from_interleaved(&[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254]);
-	/// let result = vector.plain_fft().expect("Ignoring error handling in examples");
-	/// let actual = result.data();
-	/// let expected = &[0.0, 0.0, 3.0, 0.0, 0.0, 0.0];
-	/// assert_eq!(actual.len(), expected.len());
-	/// for i in 0..actual.len() {
-	///		assert!((actual[i] - expected[i]).abs() < 1e-4);
-	/// }
-	/// ```
-	fn plain_fft(self) -> VecResult<Self::FreqPartner>;
+    type FreqPartner;
     
     /// Performs a Fast Fourier Transformation transforming a time domain vector
-	/// into a frequency domain vector. 
+    /// into a frequency domain vector. 
+    /// 
+    /// This version of the FFT neither applies a window nor does it scale the 
+    /// vector.
+    /// # Example
+    ///
+    /// ```
+    /// use basic_dsp::{ComplexTimeVector32, TimeDomainOperations, DataVector};
+    /// let vector = ComplexTimeVector32::from_interleaved(&[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254]);
+    /// let result = vector.plain_fft().expect("Ignoring error handling in examples");
+    /// let actual = result.data();
+    /// let expected = &[0.0, 0.0, 3.0, 0.0, 0.0, 0.0];
+    /// assert_eq!(actual.len(), expected.len());
+    /// for i in 0..actual.len() {
+    ///        assert!((actual[i] - expected[i]).abs() < 1e-4);
+    /// }
+    /// ```
+    fn plain_fft(self) -> VecResult<Self::FreqPartner>;
+    
+    /// Performs a Fast Fourier Transformation transforming a time domain vector
+    /// into a frequency domain vector. 
     /// # Unstable
     /// FFTs of real vectors are unstable.
-	/// # Example
-	///
-	/// ```
-	/// use basic_dsp::{ComplexTimeVector32, TimeDomainOperations, DataVector};
-	/// let vector = ComplexTimeVector32::from_interleaved(&[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254]);
-	/// let result = vector.fft().expect("Ignoring error handling in examples");
-	/// let actual = result.data();
-	/// let expected = &[0.0, 0.0, 0.0, 0.0, 3.0, 0.0];
-	/// assert_eq!(actual.len(), expected.len());
-	/// for i in 0..actual.len() {
-	///		assert!((actual[i] - expected[i]).abs() < 1e-4);
-	/// }
-	/// ```
+    /// # Example
+    ///
+    /// ```
+    /// use basic_dsp::{ComplexTimeVector32, TimeDomainOperations, DataVector};
+    /// let vector = ComplexTimeVector32::from_interleaved(&[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254]);
+    /// let result = vector.fft().expect("Ignoring error handling in examples");
+    /// let actual = result.data();
+    /// let expected = &[0.0, 0.0, 0.0, 0.0, 3.0, 0.0];
+    /// assert_eq!(actual.len(), expected.len());
+    /// for i in 0..actual.len() {
+    ///        assert!((actual[i] - expected[i]).abs() < 1e-4);
+    /// }
+    /// ```
     fn fft(self) -> VecResult<Self::FreqPartner>;
     
     /// Applies a FFT window and performs a Fast Fourier Transformation transforming a time domain vector
-	/// into a frequency domain vector. 
+    /// into a frequency domain vector. 
     fn windowed_fft(self, window: &WindowFunction<T>) -> VecResult<Self::FreqPartner>;
     
     /// Applies a window to the data vector.
@@ -81,7 +81,7 @@ pub trait TimeDomainOperations<T> : DataVector<T>
 /// with `VectorMustHaveAnOddLength` if `self.points()` isn't and odd number.
 pub trait SymmetricTimeDomainOperations<T> : DataVector<T> 
     where T : RealNumber {
-	type FreqPartner;
+    type FreqPartner;
     
     /// Performs a Symmetric Fast Fourier Transformation under the assumption that `self`
     /// is symmetric around the center. This assumption
@@ -133,61 +133,61 @@ pub trait SymmetricTimeDomainOperations<T> : DataVector<T>
 pub trait FrequencyDomainOperations<T> : DataVector<T> 
     where T : RealNumber {
     type ComplexTimePartner;
-	
-	/// Performs an Inverse Fast Fourier Transformation transforming a frequency domain vector
-	/// into a time domain vector.
-	/// 
-	/// This version of the IFFT neither applies a window nor does it scale the 
-	/// vector.
-	/// # Example
-	///
-	/// ```
-	/// use basic_dsp::{ComplexFreqVector32, FrequencyDomainOperations, DataVector};
-	/// let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 1.0, 0.0, 0.0, 0.0]);
-	/// let result = vector.plain_ifft().expect("Ignoring error handling in examples");
-	/// let actual = result.data();
-	/// let expected = &[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254];
-	/// assert_eq!(actual.len(), expected.len());
-	/// for i in 0..actual.len() {
-	///		assert!((actual[i] - expected[i]).abs() < 1e-4);
-	/// }
-	/// ```
-	fn plain_ifft(self) -> VecResult<Self::ComplexTimePartner>;
+    
+    /// Performs an Inverse Fast Fourier Transformation transforming a frequency domain vector
+    /// into a time domain vector.
+    /// 
+    /// This version of the IFFT neither applies a window nor does it scale the 
+    /// vector.
+    /// # Example
+    ///
+    /// ```
+    /// use basic_dsp::{ComplexFreqVector32, FrequencyDomainOperations, DataVector};
+    /// let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 1.0, 0.0, 0.0, 0.0]);
+    /// let result = vector.plain_ifft().expect("Ignoring error handling in examples");
+    /// let actual = result.data();
+    /// let expected = &[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254];
+    /// assert_eq!(actual.len(), expected.len());
+    /// for i in 0..actual.len() {
+    ///        assert!((actual[i] - expected[i]).abs() < 1e-4);
+    /// }
+    /// ```
+    fn plain_ifft(self) -> VecResult<Self::ComplexTimePartner>;
         
     /// This function mirrors the spectrum vector to transform a symmetric spectrum
-    /// into a full spectrum with the DC element at index 0 (no fft shift/swap halves).
-	///
+    /// into a full spectrum with the DC element at index 0 (no FFT shift/swap halves).
+    ///
     /// The argument indicates whether the resulting real vector should have `2*N` or `2*N-1` points.
     ///
-	/// # Example
-	///
-	/// ```
-	/// use basic_dsp::{ComplexFreqVector32, FrequencyDomainOperations, DataVector};
-	/// let vector = ComplexFreqVector32::from_interleaved(&[1.0, 2.0, 3.0, 4.0, 5.0, 6.0]);
-	/// let result = vector.mirror().expect("Ignoring error handling in examples");
-	/// assert_eq!([1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 5.0, -6.0, 3.0, -4.0], result.data());
-	/// ```
+    /// # Example
+    ///
+    /// ```
+    /// use basic_dsp::{ComplexFreqVector32, FrequencyDomainOperations, DataVector};
+    /// let vector = ComplexFreqVector32::from_interleaved(&[1.0, 2.0, 3.0, 4.0, 5.0, 6.0]);
+    /// let result = vector.mirror().expect("Ignoring error handling in examples");
+    /// assert_eq!([1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 5.0, -6.0, 3.0, -4.0], result.data());
+    /// ```
     fn mirror(self) -> VecResult<Self>;
     
     /// Performs an Inverse Fast Fourier Transformation transforming a frequency domain vector
-	/// into a time domain vector.
-	/// # Example
-	///
-	/// ```
-	/// use basic_dsp::{ComplexFreqVector32, FrequencyDomainOperations, DataVector};
-	/// let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 0.0, 0.0, 3.0, 0.0]);
-	/// let result = vector.ifft().expect("Ignoring error handling in examples");
-	/// let actual = result.data();
-	/// let expected = &[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254];
-	/// assert_eq!(actual.len(), expected.len());
-	/// for i in 0..actual.len() {
-	///		assert!((actual[i] - expected[i]).abs() < 1e-4);
-	/// }
-	/// ```
+    /// into a time domain vector.
+    /// # Example
+    ///
+    /// ```
+    /// use basic_dsp::{ComplexFreqVector32, FrequencyDomainOperations, DataVector};
+    /// let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 0.0, 0.0, 3.0, 0.0]);
+    /// let result = vector.ifft().expect("Ignoring error handling in examples");
+    /// let actual = result.data();
+    /// let expected = &[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254];
+    /// assert_eq!(actual.len(), expected.len());
+    /// for i in 0..actual.len() {
+    ///        assert!((actual[i] - expected[i]).abs() < 1e-4);
+    /// }
+    /// ```
     fn ifft(self) -> VecResult<Self::ComplexTimePartner>;
     
     /// Performs an Inverse Fast Fourier Transformation transforming a frequency domain vector
-	/// into a time domain vector and removes the FFT window.
+    /// into a time domain vector and removes the FFT window.
     fn windowed_ifft(self, window: &WindowFunction<T>) -> VecResult<Self::ComplexTimePartner>;
     
     /// Swaps vector halves after a Fourier Transformation.
@@ -198,7 +198,7 @@ pub trait FrequencyDomainOperations<T> : DataVector<T>
 }
 
 /// Defines all operations which are valid on `DataVectors` containing frequency domain data and
-/// the data is assumed to half of complex conjungate symmetric spectrum round 0 Hz where 
+/// the data is assumed to half of complex conjugate symmetric spectrum round 0 Hz where 
 /// the 0 Hz element itself is real.
 /// # Failures
 /// All operations in this trait fail with `VectorMustBeInFrquencyDomain`
@@ -215,7 +215,7 @@ pub trait SymmetricFrequencyDomainOperations<T> : DataVector<T>
     /// The argument indicates whether the resulting real vector should have `2*N` or `2*N-1` points.
     ///
     /// This version of the IFFT neither applies a window nor does it scale the 
-	/// vector.
+    /// vector.
     fn plain_sifft(self) -> VecResult<Self::RealTimePartner>;
     
     /// Performs a Symmetric Inverse Fast Fourier Transformation under the assumption that `self`
@@ -290,8 +290,8 @@ macro_rules! implement_window_function {
 
 macro_rules! add_time_freq_impl {
     ($($data_type:ident);*)
-	 =>
-	 {	 
+     =>
+     {     
         $(
             impl TimeDomainOperations<$data_type> for GenericDataVector<$data_type> {
                 type FreqPartner = GenericDataVector<$data_type>;
@@ -539,12 +539,12 @@ add_time_freq_impl!(f32; f64);
 
 #[cfg(test)]
 mod tests {
-	use super::*;
+    use super::*;
     use super::super::{
         ComplexTimeVector32,
         ComplexFreqVector32,
         RealTimeVector32,
-        ComplexVectorOperations,
+        ComplexVectorOps,
         DataVector};
     use num::complex::Complex32;
     use window_functions::*;
@@ -561,50 +561,50 @@ mod tests {
         }
     }
 
-	#[test]
-	fn plain_fft_test()
-	{
-		let vector = ComplexTimeVector32::from_interleaved(&[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254]);
+    #[test]
+    fn plain_fft_test()
+    {
+        let vector = ComplexTimeVector32::from_interleaved(&[1.0, 0.0, -0.5, 0.8660254, -0.5, -0.8660254]);
         let result = vector.plain_fft().unwrap();
         let expected = &[0.0, 0.0, 3.0, 0.0, 0.0, 0.0];
         let result = result.data();
         for i in 0..result.len() {
             assert!((result[i] - expected[i]).abs() < 1e-4);
         }
-	}
+    }
     
     #[test]
-	fn plain_ifft_test()
-	{
-		let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 3.0, 0.0, 0.0, 0.0]);
+    fn plain_ifft_test()
+    {
+        let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 3.0, 0.0, 0.0, 0.0]);
         let result = vector.plain_ifft().unwrap();
         let expected = &[3.0, 0.0, -1.5, 2.598076, -1.5, -2.598076];
         let result = result.data();
         for i in 0..result.len() {
             assert!((result[i] - expected[i]).abs() < 1e-4);
         }
-	}
+    }
     
     #[test]
-	fn plain_sfft_test()
-	{
-		let vector = RealTimeVector32::from_array(&[1.0, -0.5, -0.5]);
+    fn plain_sfft_test()
+    {
+        let vector = RealTimeVector32::from_array(&[1.0, -0.5, -0.5]);
         let result = vector.plain_sfft().unwrap();
         let expected = &[0.0, 0.0, 1.5, 0.0];
         let result = result.data();
         for i in 0..result.len() {
             assert!((result[i] - expected[i]).abs() < 1e-4);
         }
-	}
+    }
     
     #[test]
-	fn mirror_test()
-	{
-		let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 1.5, 0.0]);
+    fn mirror_test()
+    {
+        let vector = ComplexFreqVector32::from_interleaved(&[0.0, 0.0, 1.5, 0.0]);
         let result = vector.mirror().unwrap();
         let expected = &[0.0, 0.0, 1.5, 0.0, 1.5, 0.0];
         assert_eq!(result.data(), expected);
-	}
+    }
     
     #[test]
     fn plain_sifft_test() {
@@ -619,65 +619,65 @@ mod tests {
     }
     
     #[test]
-	fn sfft_test()
-	{
-		let vector = RealTimeVector32::from_array(&[1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0]);
+    fn sfft_test()
+    {
+        let vector = RealTimeVector32::from_array(&[1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0]);
         let result = vector.sfft().unwrap();
         let expected = &[28.0, 0.0, -3.5, 7.2678, -3.5000, 2.7912, -3.5000, 0.7989];
         let result = result.data();
         for i in 0..result.len() {
             assert!((result[i] - expected[i]).abs() < 1e-4);
         }
-	}
+    }
     
     #[test]
-	fn ifft_shift_test()
-	{
-		let mut a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0];
-		let c = ComplexFreqVector32::from_interleaved(&mut a);
+    fn ifft_shift_test()
+    {
+        let mut a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0];
+        let c = ComplexFreqVector32::from_interleaved(&mut a);
         let r = c.ifft_shift().unwrap();
-		assert_eq!(r.data(), &[3.0, 4.0, 5.0, 6.0, 1.0, 2.0]);
-	}
+        assert_eq!(r.data(), &[3.0, 4.0, 5.0, 6.0, 1.0, 2.0]);
+    }
     
     #[test]
-	fn fft_shift_test()
-	{
-		let mut a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0];
-		let c = ComplexFreqVector32::from_interleaved(&mut a);
+    fn fft_shift_test()
+    {
+        let mut a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0];
+        let c = ComplexFreqVector32::from_interleaved(&mut a);
         let r = c.fft_shift().unwrap();
-		assert_eq!(r.data(), &[5.0, 6.0, 1.0, 2.0, 3.0, 4.0]);
-	}
+        assert_eq!(r.data(), &[5.0, 6.0, 1.0, 2.0, 3.0, 4.0]);
+    }
     
     #[test]
-	fn window_test()
-	{
-		let c = ComplexTimeVector32::from_constant(Complex32::new(1.0, 0.0), 10);
+    fn window_test()
+    {
+        let c = ComplexTimeVector32::from_constant(Complex32::new(1.0, 0.0), 10);
         let triag = TriangularWindow;
         let r = c.apply_window(&triag).unwrap().magnitude().unwrap();
         let expected = [0.1, 0.3, 0.5, 0.7, 0.9, 0.9, 0.7, 0.5, 0.3, 0.1];
         let r = r.data();
-		assert_eq_tol(r, &expected, 1e-4);
-	}
+        assert_eq_tol(r, &expected, 1e-4);
+    }
     
     #[test]
-	fn window_odd_test()
-	{
-		let c = ComplexTimeVector32::from_constant(Complex32::new(1.0, 0.0), 9);
+    fn window_odd_test()
+    {
+        let c = ComplexTimeVector32::from_constant(Complex32::new(1.0, 0.0), 9);
         let triag = TriangularWindow;
         let r = c.apply_window(&triag).unwrap().magnitude().unwrap();
         let expected = [0.111, 0.333, 0.555, 0.777, 1.0, 0.777, 0.555, 0.333, 0.111];
         let r = r.data();
-		assert_eq_tol(r, &expected, 1e-2);
-	}
+        assert_eq_tol(r, &expected, 1e-2);
+    }
     
     #[test]
-	fn unapply_window_test()
-	{
-		let c = RealTimeVector32::from_array(&[0.1, 0.3, 0.5, 0.7, 0.9, 0.9, 0.7, 0.5, 0.3, 0.1]);
+    fn unapply_window_test()
+    {
+        let c = RealTimeVector32::from_array(&[0.1, 0.3, 0.5, 0.7, 0.9, 0.9, 0.7, 0.5, 0.3, 0.1]);
         let triag = TriangularWindow;
         let r = c.unapply_window(&triag).unwrap();
         let expected = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0];
         let r = r.data();
-		assert_eq_tol(r, &expected, 1e-2);
-	}
+        assert_eq_tol(r, &expected, 1e-2);
+    }
 }
\ No newline at end of file
